@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # kate: word-wrap off; remove-trailing-spaces all;
 #
-# Copyright (c) 2010-2017 Marcel Martin <marcel.martin@scilifelab.se>
+# Copyright (c) 2010-2018 Marcel Martin <marcel.martin@scilifelab.se>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,8 @@
 
 """
 cutadapt version %version
-Copyright (C) 2010-2017 Marcel Martin <marcel.martin@scilifelab.se>
+
+Copyright (C) 2010-2018 Marcel Martin <marcel.martin@scilifelab.se>
 
 cutadapt removes adapter sequences from high-throughput sequencing reads.
 
@@ -51,16 +52,12 @@ Marcel Martin. Cutadapt removes adapter sequences from high-throughput
 sequencing reads. EMBnet.Journal, 17(1):10-12, May 2011.
 http://dx.doi.org/10.14806/ej.17.1.200
 
-Use "cutadapt --help" to see all command-line options.
-See http://cutadapt.readthedocs.io/ for full documentation.
+Run "cutadapt --help" to see all command-line options.
+See https://cutadapt.readthedocs.io/ for full documentation.
 """
 
 from __future__ import print_function, division, absolute_import
 
-# Print a helpful error message if the extension modules cannot be imported.
-from cutadapt import check_importability
-check_importability()
-
 import sys
 import errno
 import time
@@ -75,8 +72,9 @@ from cutadapt.adapters import AdapterParser
 from cutadapt.modifiers import (LengthTagModifier, SuffixRemover, PrefixSuffixAdder,
 	DoubleEncoder, ZeroCapper, PrimerTrimmer, QualityTrimmer, UnconditionalCutter,
 	NEndTrimmer, AdapterCutter, NextseqQualityTrimmer, Shortener)
-from cutadapt.report import Statistics, print_report, redirect_standard_output
-from cutadapt.pipeline import SingleEndPipeline, PairedEndPipeline, OutputFiles, ParallelPipelineRunner, available_cpu_count
+from cutadapt.report import print_report, print_minimal_report, redirect_standard_output
+from cutadapt.pipeline import SingleEndPipeline, PairedEndPipeline, OutputFiles, ParallelPipelineRunner
+from cutadapt.utils import available_cpu_count
 from cutadapt.compat import PY3
 
 logger = logging.getLogger()
@@ -86,8 +84,13 @@ class CutadaptOptionParser(OptionParser):
 	def get_usage(self):
 		return self.usage.lstrip().replace('%version', __version__)
 
+	def error(self, msg):
+		print('Run "cutadapt --help" to see command-line options.', file=sys.stderr)
+		print('See https://cutadapt.readthedocs.io/ for full documentation.', file=sys.stderr)
+		self.exit(2, "\n%s: error: %s\n" % (self.get_prog_name(), msg))
 
-class CommandlineError(Exception):
+
+class CommandLineError(Exception):
 	pass
 
 
@@ -127,7 +130,7 @@ def get_option_parser():
 			"Ignored when reading csfasta/qual files. Default: auto-detect "
 			"from file name extension.")
 	parser.add_option('-j', '--cores', type=int, default=1,
-		help='Number of CPU cores to use. Default: %default')
+		help='Number of CPU cores to use. Use 0 to auto-detect. Default: %default')
 
 	# Hidden options
 	parser.add_option("--gc-content", type=float, default=50,  # it's a percentage
@@ -162,9 +165,9 @@ def get_option_parser():
 			"as with -a. This option is mostly for rescuing failed library "
 			"preparations - do not use if you know which end your adapter was "
 			"ligated to!")
-	group.add_option("-e", "--error-rate", type=float, default=0.1,
-		help="Maximum allowed error rate (no. of errors divided by the length "
-			"of the matching region). Default: %default")
+	group.add_option("-e", "--error-rate", type=float, default=0.1, metavar="RATE",
+		help="Maximum allowed error rate as value between 0 and 1 (no. of "
+			"errors divided by length of matching region). Default: %default (=10%)")
 	group.add_option("--no-indels", action='store_false', dest='indels', default=True,
 		help="Allow only mismatches in alignments. "
 			"Default: allow both mismatches and indels")
@@ -178,11 +181,15 @@ def get_option_parser():
 	group.add_option("-N", "--no-match-adapter-wildcards", action="store_false",
 		default=True, dest='match_adapter_wildcards',
 		help="Do not interpret IUPAC wildcards in adapters.")
-	group.add_option("--no-trim", dest='action', action='store_const', const=None,
-		help="Match and redirect reads to output/untrimmed-output as usual, "
-			"but do not remove adapters.")
+	group.add_option("--action", choices=('mask', 'trim', 'none'),
+		help="What to do with found adapters. trim: remove; "
+			"mask: replace with 'N' characters; "
+			"none: leave unchanged (useful with "
+			"--discard-untrimmed). Default: trim")
+	group.add_option("--no-trim", dest='action', action='store_const', const='none',
+		help="Deprecated synonym for --action=none")
 	group.add_option("--mask-adapter", dest='action', action='store_const', const='mask',
-		help="Mask adapters with 'N' characters instead of trimming them.")
+		help="Deprecated synonym for --action=mask")
 	parser.add_option_group(group)
 
 	group = OptionGroup(parser, "Additional read modifications")
@@ -201,12 +208,13 @@ def get_option_parser():
 			"value is given, only the 3' end is trimmed. If two "
 			"comma-separated cutoffs are given, the 5' end is trimmed with "
 			"the first cutoff, the 3' end with the second.")
-	group.add_option("--quality-base", type=int, default=33,
+	group.add_option("--quality-base", type=int, default=33, metavar='N',
 		help="Assume that quality values in FASTQ are encoded as ascii(quality "
-			"+ QUALITY_BASE). This needs to be set to 64 for some old Illumina "
+			"+ N). This needs to be set to 64 for some old Illumina "
 			"FASTQ files. Default: %default")
 	group.add_option("--length", "-l", type=int, default=None, metavar="LENGTH",
-			help="Shorten reads to LENGTH. This and the following modifications "
+			help="Shorten reads to LENGTH. Positive values remove bases at the end "
+			"while negative ones remove bases at the beginning. This and the following modifications "
 			"are applied after adapter trimming.")
 	group.add_option("--trim-n", action='store_true', default=False,
 		help="Trim N's on ends of reads.")
@@ -227,11 +235,11 @@ def get_option_parser():
 		description="Filters are applied after above read modifications. "
 			"Paired-end reads are always discarded pairwise (see also "
 			"--pair-filter).")
-	group.add_option("-m", "--minimum-length", type=int, default=0, metavar="LENGTH",
-		help="Discard reads shorter than LENGTH. Default: 0")
-	group.add_option("-M", "--maximum-length", type=int, default=sys.maxsize, metavar="LENGTH",
-		help="Discard reads longer than LENGTH. Default: no limit")
-	group.add_option("--max-n", type=float, default=-1.0, metavar="COUNT",
+	group.add_option("-m", "--minimum-length", default=None, metavar="LEN[:LEN2]",
+		help="Discard reads shorter than LEN. Default: 0")
+	group.add_option("-M", "--maximum-length", default=None, metavar="LEN[:LEN2]",
+		help="Discard reads longer than LEN. Default: no limit")
+	group.add_option("--max-n", type=float, default=None, metavar="COUNT",
 		help="Discard reads with more than COUNT 'N' bases. If COUNT is a number "
 			"between 0 and 1, it is interpreted as a fraction of the read length.")
 	group.add_option("--discard-trimmed", "--discard", action='store_true', default=False,
@@ -239,11 +247,15 @@ def get_option_parser():
 			"discarding too many randomly matching reads!")
 	group.add_option("--discard-untrimmed", "--trimmed-only", action='store_true', default=False,
 		help="Discard reads that do not contain an adapter.")
+	group.add_option("--discard-casava", action='store_true', default=False,
+		help="Discard reads that did not pass CASAVA filtering (header has :Y:).")
 	parser.add_option_group(group)
 
 	group = OptionGroup(parser, "Output")
 	group.add_option("--quiet", default=False, action='store_true',
 		help="Print only error messages.")
+	group.add_option("--report", choices=('full', 'minimal'), default=None,
+		help="Which type of report to print. Default: full")
 	group.add_option("-o", "--output", metavar="FILE",
 		help="Write trimmed reads to FILE. FASTQ or FASTA format is chosen "
 			"depending on input. The summary report is sent to standard output. "
@@ -300,13 +312,13 @@ def get_option_parser():
 	group.add_option("-B", dest='anywhere2', action='append', default=[], metavar='ADAPTER',
 		help="5'/3 adapter to be removed from second read in a pair.")
 	group.add_option("-U", dest='cut2', action='append', default=[], type=int, metavar="LENGTH",
-		help="Remove LENGTH bases from second read in a pair (see --cut).")
+		help="Remove LENGTH bases from second read in a pair.")
 	group.add_option("-p", "--paired-output", metavar="FILE",
 		help="Write second read in a pair to FILE.")
 	# Setting the default for pair_filter to None allows us to find out whether
 	# the option was used at all.
-	group.add_option("--pair-filter", metavar='(any|both)', default=None,
-		choices=("any", "both"),
+	group.add_option("--pair-filter", metavar='(any|both|first)', default=None,
+		choices=("any", "both", "first"),
 		help="Which of the reads in a paired-end read have to match the "
 			"filtering criterion in order for the pair to be filtered. "
 			"Default: any")
@@ -314,15 +326,14 @@ def get_option_parser():
 		help="Read and write interleaved paired-end reads.")
 	group.add_option("--untrimmed-paired-output", metavar="FILE",
 		help="Write second read in a pair to this FILE when no adapter "
-			"was found in the first read. Use this option together with "
-			"--untrimmed-output when trimming paired-end reads. Default: output "
+			"was found. Use with --untrimmed-output. Default: output "
 			"to same file as trimmed reads")
 	group.add_option("--too-short-paired-output", metavar="FILE", default=None,
 		help="Write second read in a pair to this file if pair is too short. "
-			"Use together with --too-short-output.")
+			"Use also --too-short-output.")
 	group.add_option("--too-long-paired-output", metavar="FILE", default=None,
 		help="Write second read in a pair to this file if pair is too long. "
-			"Use together with --too-long-output.")
+			"Use also --too-long-output.")
 	parser.add_option_group(group)
 
 	return parser
@@ -335,18 +346,42 @@ def parse_cutoffs(s):
 		try:
 			cutoffs = [0, int(cutoffs[0])]
 		except ValueError as e:
-			raise CommandlineError("Quality cutoff value not recognized: {0}".format(e))
+			raise CommandLineError("Quality cutoff value not recognized: {0}".format(e))
 	elif len(cutoffs) == 2:
 		try:
 			cutoffs = [int(cutoffs[0]), int(cutoffs[1])]
 		except ValueError as e:
-			raise CommandlineError("Quality cutoff value not recognized: {0}".format(e))
+			raise CommandLineError("Quality cutoff value not recognized: {0}".format(e))
 	else:
-		raise CommandlineError("Expected one value or two values separated by comma for "
+		raise CommandLineError("Expected one value or two values separated by comma for "
 			"the quality cutoff")
 	return cutoffs
 
 
+def parse_lengths(s):
+	"""Parse [INT][:[INT]] into a pair of integers. If a value is omitted, use None
+
+	>>> parse_lengths('25')
+	(25,)
+	>>> parse_lengths('17:25')
+	(17, 25)
+	>>> parse_lengths('25:')
+	(25, None)
+	>>> parse_lengths(':25')
+	(None, 25)
+	"""
+	fields = s.split(':')
+	if len(fields) not in (1, 2):
+		raise CommandLineError("Only at most one colon is allowed")
+	try:
+		values = tuple(int(f) if f != '' else None for f in fields)
+	except ValueError as e:
+		raise CommandLineError("Value not recognized: {0}".format(e))
+	if len(values) == 2 and values[0] is None and values[1] is None:
+		raise CommandLineError("Cannot parse {!r}: At least one length needs to be given".format(s))
+	return tuple(values)
+
+
 def open_output_files(options, default_outfile, interleaved):
 	"""
 	Return an OutputFiles instance. If demultiplex is True, the untrimmed, untrimmed2, out and out2
@@ -369,26 +404,26 @@ def open_output_files(options, default_outfile, interleaved):
 		return file1, file2
 
 	too_short = too_short2 = None
-	if options.minimum_length > 0:
+	if options.minimum_length is not None:
 		too_short, too_short2 = open2(options.too_short_output, options.too_short_paired_output)
 
 	too_long = too_long2 = None
-	if options.maximum_length < sys.maxsize:
+	if options.maximum_length is not None:
 		too_long, too_long2 = open2(options.too_long_output, options.too_long_paired_output)
 
 	if int(options.discard_trimmed) + int(options.discard_untrimmed) + int(
 					options.untrimmed_output is not None) > 1:
-		raise CommandlineError("Only one of the --discard-trimmed, --discard-untrimmed "
+		raise CommandLineError("Only one of the --discard-trimmed, --discard-untrimmed "
 			"and --untrimmed-output options can be used at the same time.")
 
 	demultiplex = options.output is not None and '{name}' in options.output
 	if options.paired_output is not None and (demultiplex != ('{name}' in options.paired_output)):
-		raise CommandlineError('When demultiplexing paired-end data, "{name}" must appear in '
+		raise CommandLineError('When demultiplexing paired-end data, "{name}" must appear in '
 			'both output file names (-o and -p)')
 
 	if demultiplex:
 		if options.discard_trimmed:
-			raise CommandlineError("Do not use --discard-trimmed when demultiplexing.")
+			raise CommandLineError("Do not use --discard-trimmed when demultiplexing.")
 
 		out = options.output
 		untrimmed = options.output.replace('{name}', 'unknown')
@@ -466,7 +501,7 @@ def determine_interleaved(options, args):
 		is_interleaved_input = len(args) == 1
 		is_interleaved_output = not options.paired_output
 		if not is_interleaved_input and not is_interleaved_output:
-			raise CommandlineError("When --interleaved is used, you cannot provide both two "
+			raise CommandLineError("When --interleaved is used, you cannot provide both two "
 				"input files and two output files")
 	return is_interleaved_input, is_interleaved_output
 
@@ -476,14 +511,14 @@ def input_files_from_parsed_args(args, paired, interleaved):
 	Return tuple (input_filename, input_paired_filename, quality_filename)
 	"""
 	if len(args) == 0:
-		raise CommandlineError("At least one parameter needed: name of a FASTA or FASTQ file.")
+		raise CommandLineError("Please give me something to do!")
 	elif len(args) > 2:
-		raise CommandlineError("Too many parameters.")
+		raise CommandLineError("Too many parameters.")
 	input_filename = args[0]
 	if input_filename.endswith('.qual'):
-		raise CommandlineError("If a .qual file is given, it must be the second argument.")
+		raise CommandLineError("If a .qual file is given, it must be the second argument.")
 	if paired and len(args) == 1 and not interleaved:
-		raise CommandlineError("When paired-end trimming is enabled via -A/-G/-B/-U/"
+		raise CommandLineError("When paired-end trimming is enabled via -A/-G/-B/-U/"
 			"--interleaved or -p, two input files are required.")
 
 	input_paired_filename = None
@@ -501,41 +536,41 @@ def pipeline_from_parsed_args(options, paired, pair_filter_mode, quality_filenam
 	"""
 	Setup a processing pipeline from parsed command-line options.
 
-	If there are any problems parsing the arguments, a CommandlineError is thrown.
+	If there are any problems parsing the arguments, a CommandLineError is thrown.
 
 	Return an instance of Pipeline (SingleEndPipeline or PairedEndPipeline)
 	"""
 
 	if not paired:
 		if options.untrimmed_paired_output:
-			raise CommandlineError("Option --untrimmed-paired-output can only be used when "
+			raise CommandLineError("Option --untrimmed-paired-output can only be used when "
 				"trimming paired-end reads (with option -p).")
 
 	if paired:
 		if not is_interleaved_output:
 			if not options.paired_output:
-				raise CommandlineError("When paired-end trimming is enabled via -A/-G/-B/-U, "
+				raise CommandLineError("When paired-end trimming is enabled via -A/-G/-B/-U, "
 					"a second output file needs to be specified via -p (--paired-output).")
 			if not options.output:
-				raise CommandlineError("When you use -p or --paired-output, you must also "
+				raise CommandLineError("When you use -p or --paired-output, you must also "
 					"use the -o option.")
 
 		if bool(options.untrimmed_output) != bool(options.untrimmed_paired_output):
-			raise CommandlineError("When trimming paired-end reads, you must use either none "
+			raise CommandLineError("When trimming paired-end reads, you must use either none "
 				"or both of the --untrimmed-output/--untrimmed-paired-output options.")
 		if options.too_short_output and not options.too_short_paired_output:
-			raise CommandlineError("When using --too-short-output with paired-end "
+			raise CommandLineError("When using --too-short-output with paired-end "
 				"reads, you also need to use --too-short-paired-output")
 		if options.too_long_output and not options.too_long_paired_output:
-			raise CommandlineError("When using --too-long-output with paired-end "
+			raise CommandLineError("When using --too-long-output with paired-end "
 				"reads, you also need to use --too-long-paired-output")
 	elif quality_filename is not None:
 		if options.format is not None:
-			raise CommandlineError('If a pair of .fasta and .qual files is given, the -f/--format '
+			raise CommandLineError('If a pair of .fasta and .qual files is given, the -f/--format '
 				'parameter cannot be used.')
 
 	if options.format is not None and options.format.lower() not in ['fasta', 'fastq', 'sra-fastq']:
-		raise CommandlineError("The input file format must be either 'fasta', 'fastq' or "
+		raise CommandLineError("The input file format must be either 'fasta', 'fastq' or "
 			"'sra-fastq' (not '{0}').".format(options.format))
 
 	if options.maq:
@@ -547,22 +582,24 @@ def pipeline_from_parsed_args(options, paired, pair_filter_mode, quality_filenam
 	if options.zero_cap is None:
 		options.zero_cap = options.colorspace
 	if options.trim_primer and not options.colorspace:
-		raise CommandlineError("Trimming the primer makes only sense in colorspace.")
+		raise CommandLineError("Trimming the primer makes only sense in colorspace.")
 	if options.double_encode and not options.colorspace:
-		raise CommandlineError("Double-encoding makes only sense in colorspace.")
+		raise CommandLineError("Double-encoding makes only sense in colorspace.")
 	if options.anywhere and options.colorspace:
-		raise CommandlineError("Using --anywhere with colorspace reads is currently not supported "
+		raise CommandLineError("Using --anywhere with colorspace reads is currently not supported "
 			"(if you think this may be useful, contact the author).")
 	if not (0 <= options.error_rate <= 1.):
-		raise CommandlineError("The maximum error rate must be between 0 and 1.")
+		raise CommandLineError("The maximum error rate must be between 0 and 1.")
 	if options.overlap < 1:
-		raise CommandlineError("The overlap must be at least 1.")
+		raise CommandLineError("The overlap must be at least 1.")
 	if not (0 <= options.gc_content <= 100):
-		raise CommandlineError("GC content must be given as percentage between 0 and 100")
+		raise CommandLineError("GC content must be given as percentage between 0 and 100")
+	if options.action == 'none':
+		options.action = None
 
 	if options.colorspace:
 		if options.match_read_wildcards:
-			raise CommandlineError('IUPAC wildcards not supported in colorspace')
+			raise CommandLineError('IUPAC wildcards not supported in colorspace')
 		options.match_adapter_wildcards = False
 
 	adapter_parser = AdapterParser(
@@ -578,10 +615,10 @@ def pipeline_from_parsed_args(options, paired, pair_filter_mode, quality_filenam
 		adapters2 = adapter_parser.parse_multi(options.adapters2, options.anywhere2, options.front2)
 	except IOError as e:
 		if e.errno == errno.ENOENT:
-			raise CommandlineError(e)
+			raise CommandLineError(e)
 		raise
 	except ValueError as e:
-		raise CommandlineError(e)
+		raise CommandLineError(e)
 	if options.debug:
 		for adapter in adapters + adapters2:
 			adapter.enable_debug()
@@ -596,18 +633,18 @@ def pipeline_from_parsed_args(options, paired, pair_filter_mode, quality_filenam
 
 	if options.cut:
 		if len(options.cut) > 2:
-			raise CommandlineError("You cannot remove bases from more than two ends.")
+			raise CommandLineError("You cannot remove bases from more than two ends.")
 		if len(options.cut) == 2 and options.cut[0] * options.cut[1] > 0:
-			raise CommandlineError("You cannot remove bases from the same end twice.")
+			raise CommandLineError("You cannot remove bases from the same end twice.")
 		for cut in options.cut:
 			if cut != 0:
 				pipeline.add1(UnconditionalCutter(cut))
 
 	if options.cut2:
 		if len(options.cut2) > 2:
-			raise CommandlineError("You cannot remove bases from more than two ends.")
+			raise CommandLineError("You cannot remove bases from more than two ends.")
 		if len(options.cut2) == 2 and options.cut2[0] * options.cut2[1] > 0:
-			raise CommandlineError("You cannot remove bases from the same end twice.")
+			raise CommandLineError("You cannot remove bases from the same end twice.")
 		for cut in options.cut2:
 			if cut != 0:
 				pipeline.add2(UnconditionalCutter(cut))
@@ -621,11 +658,9 @@ def pipeline_from_parsed_args(options, paired, pair_filter_mode, quality_filenam
 	if adapters:
 		adapter_cutter = AdapterCutter(adapters, options.times, options.action)
 		pipeline.add1(adapter_cutter)
-		pipeline.n_adapters += len(adapters)
 	if adapters2:
 		adapter_cutter2 = AdapterCutter(adapters2, options.times, options.action)
 		pipeline.add2(adapter_cutter2)
-		pipeline.n_adapters += len(adapters2)
 
 	# Modifiers that apply to both reads of paired-end reads unless in legacy mode
 	if options.length is not None:
@@ -645,7 +680,23 @@ def pipeline_from_parsed_args(options, paired, pair_filter_mode, quality_filenam
 	if options.zero_cap:
 		pipeline.add(ZeroCapper(quality_base=options.quality_base))
 	if options.trim_primer:
-		pipeline.add(PrimerTrimmer)
+		pipeline.add(PrimerTrimmer())
+
+	# Set filtering parameters
+	# Minimum/maximum length
+	for attr in 'minimum_length', 'maximum_length':
+		param = getattr(options, attr)
+		if param is not None:
+			lengths = parse_lengths(param)
+			if not paired and len(lengths) == 2:
+				raise CommandLineError('Two minimum or maximum lengths given for single-end data')
+			if paired and len(lengths) == 1:
+				lengths = (lengths[0], lengths[0])
+			setattr(pipeline, attr, lengths)
+	pipeline.max_n = options.max_n
+	pipeline.discard_casava = options.discard_casava
+	pipeline.discard_trimmed = options.discard_trimmed
+	pipeline.discard_untrimmed = options.discard_untrimmed
 
 	return pipeline
 
@@ -665,7 +716,9 @@ def main(cmdlineargs=None, default_outfile=sys.stdout):
 	# Setup logging only if there are not already any handlers (can happen when
 	# this function is being called externally such as from unit tests)
 	if not logging.root.handlers:
-		setup_logging(stdout=bool(options.output), quiet=options.quiet)
+		setup_logging(stdout=bool(options.output), quiet=options.quiet or options.report == 'minimal')
+	if options.quiet and options.report:
+		parser.error("Options --quiet and --report cannot be used at the same time")
 
 	paired = determine_paired_mode(options)
 	assert paired in (False, 'first', 'both')
@@ -687,11 +740,13 @@ def main(cmdlineargs=None, default_outfile=sys.stdout):
 			paired, is_interleaved_input)
 		pipeline = pipeline_from_parsed_args(options, paired, pair_filter_mode, quality_filename, is_interleaved_output)
 		outfiles = open_output_files(options, default_outfile, is_interleaved_output)
-	except CommandlineError as e:
+	except CommandLineError as e:
 		parser.error(e)
 		return  # avoid IDE warnings below
 
-	cores = max(1, options.cores)
+	if options.cores < 0:
+		parser.error('Value for --cores cannot be negative')
+	cores = available_cpu_count() if options.cores == 0 else options.cores
 	if cores > 1:
 		if (
 			PY3
@@ -699,7 +754,6 @@ def main(cmdlineargs=None, default_outfile=sys.stdout):
 			and quality_filename is None
 			and not options.colorspace
 			and options.format is None
-			and cores > 1
 		):
 			runner = ParallelPipelineRunner(pipeline, cores, options.buffer_size)
 		else:
@@ -720,8 +774,7 @@ def main(cmdlineargs=None, default_outfile=sys.stdout):
 		runner.set_input(input_filename, file2=input_paired_filename,
 			qualfile=quality_filename, colorspace=options.colorspace,
 			fileformat=options.format, interleaved=is_interleaved_input)
-		runner.set_output(outfiles, options.minimum_length, options.maximum_length, options.max_n,
-			options.discard_trimmed, options.discard_untrimmed)
+		runner.set_output(outfiles)
 	except (seqio.UnknownFileType, IOError) as e:
 		parser.error(e)
 
@@ -730,10 +783,8 @@ def main(cmdlineargs=None, default_outfile=sys.stdout):
 	logger.info("This is cutadapt %s with Python %s%s", __version__,
 		platform.python_version(), opt)
 	logger.info("Command line parameters: %s", " ".join(cmdlineargs))
-	logger.info('Running on %d core%s', cores, 's' if cores > 1 else '')
-	logger.info("Trimming %s adapter%s with at most %.1f%% errors in %s mode ...",
-		pipeline.n_adapters, 's' if pipeline.n_adapters != 1 else '',
-		options.error_rate * 100,
+	logger.info("Processing reads on %d core%s in %s mode ...",
+		cores, 's' if cores > 1 else '',
 		{False: 'single-end', 'first': 'paired-end legacy', 'both': 'paired-end'}[pipeline.paired])
 
 	if pipeline.should_warn_legacy:
@@ -762,7 +813,10 @@ def main(cmdlineargs=None, default_outfile=sys.stdout):
 		# send statistics to stderr if result was sent to stdout
 		stat_file = sys.stderr if options.output is None else None
 		with redirect_standard_output(stat_file):
-			print_report(stats, elapsed, options.gc_content / 100)
+			if options.report == 'minimal':
+				print_minimal_report(stats, elapsed, options.gc_content / 100)
+			else:
+				print_report(stats, elapsed, options.gc_content / 100)
 
 
 if __name__ == '__main__':
