@@ -12,16 +12,30 @@ from cutadapt import align, colorspace
 from cutadapt.seqio import FastaReader
 
 
-# Constants for the find_best_alignment function.
+# Constants for the Aligner.locate() function.
 # The function is called with SEQ1 as the adapter, SEQ2 as the read.
 # TODO get rid of those constants, use strings instead
 BACK = align.START_WITHIN_SEQ2 | align.STOP_WITHIN_SEQ2 | align.STOP_WITHIN_SEQ1
 FRONT = align.START_WITHIN_SEQ2 | align.STOP_WITHIN_SEQ2 | align.START_WITHIN_SEQ1
 PREFIX = align.STOP_WITHIN_SEQ2
 SUFFIX = align.START_WITHIN_SEQ2
+# Just like FRONT/BACK, but without internal matches
+FRONT_NOT_INTERNAL = align.START_WITHIN_SEQ1 | align.STOP_WITHIN_SEQ2
+BACK_NOT_INTERNAL = align.START_WITHIN_SEQ2 | align.STOP_WITHIN_SEQ1
 ANYWHERE = align.SEMIGLOBAL
 LINKED = 'linked'
 
+# TODO put this in some kind of "list of pre-defined adapter types" along with the info above
+WHERE_TO_REMOVE_MAP = {
+	PREFIX: 'prefix',
+	FRONT_NOT_INTERNAL: 'prefix',
+	FRONT: 'prefix',
+	BACK: 'suffix',
+	SUFFIX: 'suffix',
+	BACK_NOT_INTERNAL: 'suffix',
+	ANYWHERE: 'auto',
+}
+
 
 def parse_braces(sequence):
 	"""
@@ -29,7 +43,7 @@ def parse_braces(sequence):
 	occurrences of x. Raise ValueError if the expression cannot be parsed.
 
 	>>> parse_braces('TGA{5}CT')
-	TGAAAAACT
+	'TGAAAAACT'
 	"""
 	# Simple DFA with four states, encoded in prev
 	result = ''
@@ -70,84 +84,225 @@ class AdapterParser(object):
 	"""
 	def __init__(self, colorspace=False, **kwargs):
 		self.colorspace = colorspace
-		self.constructor_args = kwargs
+		# kwargs: max_error_rate, min_overlap, read_wildcards, adapter_wildcards, indels
+		self.default_parameters = kwargs
 		self.adapter_class = ColorspaceAdapter if colorspace else Adapter
 
-	def _parse_no_file(self, spec, name=None, cmdline_type='back'):
+	@staticmethod
+	def _extract_name(spec):
+		"""
+		Parse an adapter specification given as 'name=adapt' into 'name' and 'adapt'.
+		"""
+		fields = spec.split('=', 1)
+		if len(fields) > 1:
+			name, spec = fields
+			name = name.strip()
+		else:
+			name = None
+		spec = spec.strip()
+		return name, spec
+
+	parameters = {
+		# abbreviations
+		'e': 'max_error_rate',
+		'error_rate': 'max_error_rate',
+		'o': 'min_overlap',
+
+		# allowed parameters
+		'max_error_rate': None,
+		'min_overlap': None,
+		'anywhere': None,
+	}
+
+	@staticmethod
+	def _parse_parameters(spec):
+		"""Parse key=value;key=value;key=value into a dict"""
+
+		fields = spec.split(';')
+		result = dict()
+		for field in fields:
+			field = field.strip()
+			if not field:
+				continue
+			key, equals, value = field.partition('=')
+			if equals == '=' and value == '':
+				raise ValueError('No value given')
+			key = key.strip()
+			if key not in AdapterParser.parameters:
+				raise KeyError('Unknown parameter {}'.format(key))
+			# unabbreviate
+			while AdapterParser.parameters[key] is not None:
+				key = AdapterParser.parameters[key]
+			value = value.strip()
+			if value == '':
+				value = True
+			else:
+				try:
+					value = int(value)
+				except ValueError:
+					value = float(value)
+			if key in result:
+				raise KeyError('Key {} specified twice'.format(key))
+			result[key] = value
+		return result
+
+	@staticmethod
+	def _parse_not_linked(spec, cmdline_type):
+		"""
+		Parse an adapter specification for a non-linked adapter (without '...')
+
+		Allow:
+		'back' and ADAPTER
+		'back' and ADAPTERX
+		'back' and ADAPTER$
+		'front' and ADAPTER
+		'front' and XADAPTER
+		'front' and ^ADAPTER
+		'anywhere' and ADAPTER
+		"""
+		error = ValueError(
+				"You cannot use multiple placement restrictions for an adapter at the same time. "
+				"Choose one of ^ADAPTER, ADAPTER$, XADAPTER or ADAPTERX")
+		spec, middle, parameters_spec = spec.partition(';')
+		name, spec = AdapterParser._extract_name(spec)
+		spec = spec.strip()
+
+		parameters = AdapterParser._parse_parameters(parameters_spec)
+
+		# Special case for adapters consisting of only X characters:
+		# This needs to be supported for backwards-compatibilitity
+		if len(spec.strip('X')) == 0:
+			return name, None, spec, None, {}
+
+		front_restriction = None
+		if spec.startswith('^'):
+			front_restriction = 'anchored'
+			spec = spec[1:]
+		if spec.upper().startswith('X'):
+			if front_restriction is not None:
+				raise error
+			front_restriction = 'noninternal'
+			spec = spec.lstrip('xX')
+
+		back_restriction = None
+		if spec.endswith('$'):
+			back_restriction = 'anchored'
+			spec = spec[:-1]
+		if spec.upper().endswith('X'):
+			if back_restriction is not None:
+				raise error
+			back_restriction = 'noninternal'
+			spec = spec.rstrip('xX')
+
+		n_placement_restrictions = int(bool(front_restriction)) + int(bool(back_restriction))
+		if n_placement_restrictions > 1:
+			raise error
+
+		if cmdline_type == 'front' and back_restriction:
+			raise ValueError(
+				"Allowed placement restrictions for a 5' adapter are XADAPTER and ^ADAPTER")
+		if cmdline_type == 'back' and front_restriction:
+			raise ValueError(
+				"Allowed placement restrictions for a 3' adapter are ADAPTERX and ADAPTER$")
+
+		if cmdline_type == 'anywhere' and n_placement_restrictions > 0:
+			raise ValueError(
+				"Placement restrictions (with X, ^, $) not supported for 'anywhere' (-b) adapters")
+		assert front_restriction is None or back_restriction is None
+		return name, front_restriction, spec, back_restriction, parameters
+
+	def _parse(self, spec, cmdline_type='back', name=None):
 		"""
 		Parse an adapter specification not using ``file:`` notation and return
-		an object of an appropriate Adapter class. The notation for anchored
-		5' and 3' adapters is supported. If the name parameter is None, then
-		an attempt is made to extract the name from the specification
-		(If spec is 'name=ADAPTER', name will be 'name'.)
+		an object of an appropriate Adapter class.
+
+		name -- Adapter name if not included as part of the spec. (If spec is
+		'name=ADAPTER', name will be 'name'.)
 
 		cmdline_type -- describes which commandline parameter was used (``-a``
 		is 'back', ``-b`` is 'anywhere', and ``-g`` is 'front').
 		"""
-		if name is None:
-			name, spec = self._extract_name(spec)
-		orig_spec = spec
-		types = dict(back=BACK, front=FRONT, anywhere=ANYWHERE)
-		if cmdline_type not in types:
-			raise ValueError('cmdline_type cannot be {0!r}'.format(cmdline_type))
-		where = types[cmdline_type]
+		if cmdline_type not in ('front', 'back', 'anywhere'):
+			raise ValueError('cmdline_type cannot be {!r}'.format(cmdline_type))
+		spec1, middle, spec2 = spec.partition('...')
+		del spec
 
-		front_anchored, back_anchored = False, False
-		if spec.startswith('^'):
-			spec = spec[1:]
-			front_anchored = True
-		if spec.endswith('$'):
-			spec = spec[:-1]
-			back_anchored = True
-
-		sequence1, middle, sequence2 = spec.partition('...')
-		if where == ANYWHERE:
-			if front_anchored or back_anchored:
-				raise ValueError("'anywhere' (-b) adapters may not be anchored")
-			if middle == '...':
+		# Handle linked adapter
+		if middle == '...' and spec1 and spec2:
+			if cmdline_type == 'anywhere':
 				raise ValueError("'anywhere' (-b) adapters may not be linked")
-			return self.adapter_class(sequence=spec, where=where, name=name, **self.constructor_args)
+			if self.colorspace:
+				raise NotImplementedError(
+					'Using linked adapters in colorspace is not supported')
+			name1, front1, sequence1, back1, parameters1 = self._parse_not_linked(spec1, 'front')
+			name2, front2, sequence2, back2, parameters2 = self._parse_not_linked(spec2, 'back')
+			if not name:
+				name = name1
+
+			# Automatically anchor the 5' adapter if -a is used
+			if cmdline_type == 'back' and front1 is None:
+				front1 = 'anchored'
+
+			front_anchored = front1 == 'anchored'
+			back_anchored = back2 == 'anchored'
+			require_both = True if not front_anchored and not back_anchored else None
+			front_parameters = self.default_parameters.copy()
+			front_parameters.update(parameters1)
+			back_parameters = self.default_parameters.copy()
+			back_parameters.update(parameters2)
+			return LinkedAdapter(
+				sequence1, sequence2, name=name,
+				front_restriction=front1,
+				back_restriction=back2,
+				require_both=require_both,
+				front_parameters=front_parameters,
+				back_parameters=back_parameters)
 
-		assert where == FRONT or where == BACK
 		if middle == '...':
-			if not sequence1:
-				if where == BACK:  # -a ...ADAPTER
-					spec = sequence2
+			if not spec1:
+				if cmdline_type == 'back':  # -a ...ADAPTER
+					spec = spec2
 				else:  # -g ...ADAPTER
 					raise ValueError('Invalid adapter specification')
-			elif not sequence2:
-				if where == BACK:  # -a ADAPTER...
-					spec = sequence1
-					where = FRONT
-					front_anchored = True
+			elif not spec2:
+				if cmdline_type == 'back':  # -a ADAPTER...
+					cmdline_type = 'front'
+					spec = '^' + spec1
 				else:  # -g ADAPTER...
-					spec = sequence1
+					spec = spec1
 			else:
-				# linked adapter
-				if self.colorspace:
-					raise NotImplementedError(
-						'Using linked adapters in colorspace is not supported')
-				# automatically anchor 5' adapter if -a is used
-				if where == BACK:
-					front_anchored = True
+				assert False, 'This should not happen'
+		else:
+			spec = spec1
 
-				require_both = True if not front_anchored and not back_anchored else None
-				return LinkedAdapter(sequence1, sequence2, name=name,
-					front_anchored=front_anchored, back_anchored=back_anchored,
-					require_both=require_both,
-					**self.constructor_args)
-		if front_anchored and back_anchored:
-			raise ValueError('Trying to use both "^" and "$" in adapter specification {!r}'.format(orig_spec))
-		if front_anchored:
-			if where == BACK:
-				raise ValueError("Cannot anchor the 3' adapter at its 5' end")
+		specname, front_restriction, sequence, back_restriction, parameters = self._parse_not_linked(
+			spec, cmdline_type)
+		del spec
+		if front_restriction == 'anchored':
 			where = PREFIX
-		elif back_anchored:
-			if where == FRONT:
-				raise ValueError("Cannot anchor 5' adapter at 3' end")
+		elif front_restriction == 'noninternal':
+			where = FRONT_NOT_INTERNAL
+		elif back_restriction == 'anchored':
 			where = SUFFIX
+		elif back_restriction == 'noninternal':
+			where = BACK_NOT_INTERNAL
+		elif cmdline_type == 'front':
+			where = FRONT
+		elif cmdline_type == 'back':
+			where = BACK
+		else:
+			assert cmdline_type == 'anywhere'
+			where = ANYWHERE
 
-		return self.adapter_class(sequence=spec, where=where, name=name, **self.constructor_args)
+		if not name:
+			name = specname
+		if parameters.get('anywhere', False):
+			parameters['remove'] = WHERE_TO_REMOVE_MAP[where]
+			where = ANYWHERE
+			del parameters['anywhere']
+		params = self.default_parameters.copy()
+		params.update(parameters)
+		return self.adapter_class(sequence=sequence, where=where, name=name, **params)
 
 	def parse(self, spec, cmdline_type='back'):
 		"""
@@ -162,23 +317,9 @@ class AdapterParser(object):
 			with FastaReader(spec[5:]) as fasta:
 				for record in fasta:
 					name = record.name.split(None, 1)[0]
-					yield self._parse_no_file(record.sequence, name, cmdline_type)
+					yield self._parse(record.sequence, cmdline_type, name=name)
 		else:
-			name, spec = self._extract_name(spec)
-			yield self._parse_no_file(spec, name, cmdline_type)
-
-	def _extract_name(self, spec):
-		"""
-		Parse an adapter specification given as 'name=adapt' into 'name' and 'adapt'.
-		"""
-		fields = spec.split('=', 1)
-		if len(fields) > 1:
-			name, spec = fields
-			name = name.strip()
-		else:
-			name = None
-		spec = spec.strip()
-		return name, spec
+			yield self._parse(spec, cmdline_type, name=None)
 
 	def parse_multi(self, back, anywhere, front):
 		"""
@@ -212,11 +353,12 @@ class EndStatistics(object):
 		self.has_wildcards = adapter.adapter_wildcards
 		# self.errors[l][e] == n iff n times a sequence of length l matching at e errors was removed
 		self.errors = defaultdict(returns_defaultdict_int)
-		self._remove_before = adapter.remove_before
+		self._remove = adapter.remove
 		self.adjacent_bases = {'A': 0, 'C': 0, 'G': 0, 'T': 0, '': 0}
 
 	def __iadd__(self, other):
-		if self.where != other.where or self.max_error_rate != other.max_error_rate or self.sequence != other.sequence:
+		if (self.where != other.where or self._remove != other._remove or
+				self.max_error_rate != other.max_error_rate or self.sequence != other.sequence):
 			raise RuntimeError('Incompatible EndStatistics, cannot be added')
 		for base in ('A', 'C', 'G', 'T', ''):
 			self.adjacent_bases[base] += other.adjacent_bases[base]
@@ -244,7 +386,8 @@ class EndStatistics(object):
 		specify which (front or back) of the two adapters is meant.
 		"""
 		seq = self.sequence
-		if self._remove_before:
+		# FIXME this is broken for self._remove == 'auto'
+		if self._remove == 'prefix':
 			seq = seq[::-1]
 		allowed_bases = 'CGRYSKMBDHVN' if self.has_wildcards else 'GC'
 		p = 1
@@ -329,9 +472,9 @@ class Match(object):
 		If there are indels, this is not reliable as the full alignment
 		is not available.
 		"""
-		wildcards = [ self.read.sequence[self.rstart + i] for i in range(self.length)
+		wildcards = [self.read.sequence[self.rstart + i] for i in range(self.length)
 			if self.adapter.sequence[self.astart + i] == wildcard_char and
-				self.rstart + i < len(self.read.sequence) ]
+				self.rstart + i < len(self.read.sequence)]
 		return ''.join(wildcards)
 
 	def rest(self):
@@ -441,7 +584,14 @@ class Adapter(object):
 
 	where --  One of the BACK, FRONT, PREFIX, SUFFIX or ANYWHERE constants.
 		This influences where the adapter is allowed to appear within in the
-		read and also which part of the read is removed.
+		read.
+
+	remove -- describes which part of the read to remove if the adapter was found:
+		  * "prefix" (for a 3' adapter)
+		  * "suffix" (for a 5' adapter)
+		  * "auto" for a 5'/3' mixed adapter (if the match involves the first base of the read, it
+		    is assumed to be a 5' adapter and a 3' otherwise)
+		  * None: One of the above is chosen depending on the 'where' parameter
 
 	sequence -- The adapter sequence as string. Will be converted to uppercase.
 		Also, Us will be converted to Ts.
@@ -462,17 +612,19 @@ class Adapter(object):
 		unique number.
 	"""
 
-	def __init__(self, sequence, where, max_error_rate=0.1, min_overlap=3,
+	def __init__(self, sequence, where, remove=None, max_error_rate=0.1, min_overlap=3,
 			read_wildcards=False, adapter_wildcards=True, name=None, indels=True):
-		self.debug = False
+		self._debug = False
 		self.name = _generate_adapter_name() if name is None else name
 		self.sequence = parse_braces(sequence.upper().replace('U', 'T'))  # TODO move away
 		if not self.sequence:
 			raise ValueError('Sequence is empty')
 		self.where = where
+		if remove not in (None, 'prefix', 'suffix'):
+			raise ValueError('remove parameter must be "prefix", "suffix", "auto" or None')
+		self.remove = WHERE_TO_REMOVE_MAP[where] if remove is None else remove
 		self.max_error_rate = max_error_rate
 		self.min_overlap = min(min_overlap, len(self.sequence))
-		self.indels = indels
 		iupac = frozenset('XACGTURYSWKMBDHVN')
 		if adapter_wildcards and not set(self.sequence) <= iupac:
 			for c in self.sequence:
@@ -483,11 +635,11 @@ class Adapter(object):
 		# Optimization: Use non-wildcard matching if only ACGT is used
 		self.adapter_wildcards = adapter_wildcards and not set(self.sequence) <= set('ACGT')
 		self.read_wildcards = read_wildcards
-		self.remove_before = where not in (BACK, SUFFIX)
 
 		self.aligner = align.Aligner(self.sequence, self.max_error_rate,
 			flags=self.where, wildcard_ref=self.adapter_wildcards, wildcard_query=self.read_wildcards)
 		self.aligner.min_overlap = self.min_overlap
+		self.indels = indels
 		if not self.indels:
 			# TODO
 			# When indels are disallowed, an entirely different algorithm
@@ -496,7 +648,7 @@ class Adapter(object):
 
 	def __repr__(self):
 		return '<Adapter(name={name!r}, sequence={sequence!r}, where={where}, '\
-			'max_error_rate={max_error_rate}, min_overlap={min_overlap}, '\
+			'remove={remove}, max_error_rate={max_error_rate}, min_overlap={min_overlap}, '\
 			'read_wildcards={read_wildcards}, '\
 			'adapter_wildcards={adapter_wildcards}, '\
 			'indels={indels})>'.format(**vars(self))
@@ -506,7 +658,7 @@ class Adapter(object):
 		Print out the dynamic programming matrix after matching a read to an
 		adapter.
 		"""
-		self.debug = True
+		self._debug = True
 		self.aligner.enable_debug()
 
 	def match_to(self, read, match_class=Match):
@@ -518,23 +670,21 @@ class Adapter(object):
 		overlap length, maximum error rate).
 		"""
 		read_seq = read.sequence.upper()  # temporary copy
-		remove_before = self.remove_before
 		pos = -1
+
 		# try to find an exact match first unless wildcards are allowed
 		if not self.adapter_wildcards:
 			if self.where == PREFIX:
 				pos = 0 if read_seq.startswith(self.sequence) else -1
 			elif self.where == SUFFIX:
 				pos = (len(read_seq) - len(self.sequence)) if read_seq.endswith(self.sequence) else -1
-			else:
+			elif self.where == BACK or self.where == FRONT:
 				pos = read_seq.find(self.sequence)
+			# TODO BACK_NOT_INTERNAL, FRONT_NOT_INTERNAL
 		if pos >= 0:
-			if self.where == ANYWHERE:
-				# guess: if alignment starts at pos 0, it’s a 5' adapter
-				remove_before = pos == 0
-			match = match_class(
+			match_args = (
 				0, len(self.sequence), pos, pos + len(self.sequence),
-				len(self.sequence), 0, remove_before, self, read)
+				len(self.sequence), 0)
 		else:
 			# try approximate matching
 			if not self.indels and self.where in (PREFIX, SUFFIX):
@@ -546,24 +696,28 @@ class Adapter(object):
 						wildcard_ref=self.adapter_wildcards, wildcard_query=self.read_wildcards)
 				astart, astop, rstart, rstop, matches, errors = alignment
 				if astop - astart >= self.min_overlap and errors / (astop - astart) <= self.max_error_rate:
-					match = match_class(*(alignment + (remove_before, self, read)))
+					match_args = alignment
 				else:
-					match = None
+					match_args = None
 			else:
 				alignment = self.aligner.locate(read_seq)
-				if self.debug:
+				if self._debug:
 					print(self.aligner.dpmatrix)  # pragma: no cover
 				if alignment is None:
-					match = None
+					match_args = None
 				else:
 					astart, astop, rstart, rstop, matches, errors = alignment
-					if self.where == ANYWHERE:
-						# guess: if alignment starts at pos 0, it’s a 5' adapter
-						remove_before = rstart == 0
-					match = match_class(astart, astop, rstart, rstop, matches, errors, remove_before, self, read)
+					match_args = (astart, astop, rstart, rstop, matches, errors)
 
-		if match is None:
+		if match_args is None:
 			return None
+		if self.remove == 'auto':
+			# guess: if alignment starts at pos 0, it’s a 5' adapter
+			remove_before = match_args[2] == 0  # index 2 is rstart
+		else:
+			remove_before = self.remove == 'prefix'
+		match = match_class(*match_args, remove_before=remove_before, adapter=self, read=read)
+
 		assert match.length > 0 and match.errors / match.length <= self.max_error_rate, match
 		assert match.length >= self.min_overlap
 		return match
@@ -619,15 +773,15 @@ class ColorspaceAdapter(Adapter):
 		if pos >= 0:
 			match = ColorspaceMatch(
 				0, len(asequence), pos, pos + len(asequence),
-				len(asequence), 0, self.remove_before, self, read)
+				len(asequence), 0, self.remove, self, read)
 		else:
 			# try approximate matching
 			self.aligner.reference = asequence
 			alignment = self.aligner.locate(read.sequence)
-			if self.debug:
+			if self._debug:
 				print(self.aligner.dpmatrix)  # pragma: no cover
 			if alignment is not None:
-				match = ColorspaceMatch(*(alignment + (self.remove_before, self, read)))
+				match = ColorspaceMatch(*(alignment + (self.remove, self, read)))
 			else:
 				match = None
 
@@ -686,27 +840,44 @@ class LinkedMatch(object):
 class LinkedAdapter(object):
 	"""
 	"""
-	def __init__(self, front_sequence, back_sequence,
-			front_anchored=True, back_anchored=False, require_both=None, name=None, **kwargs):
+	def __init__(self, front_sequence, back_sequence, front_restriction='anchored',
+			back_restriction=None, require_both=None, name=None,
+			front_parameters=dict(), back_parameters=dict()):
 		"""
 		require_both -- require both adapters to match. If not specified, the default is to
 			require only anchored adapters to match.
 		kwargs are passed on to individual Adapter constructors
 		"""
-		where1 = PREFIX if front_anchored else FRONT
-		where2 = SUFFIX if back_anchored else BACK
+		if front_restriction is None:
+			where1 = FRONT
+		elif front_restriction == 'anchored':
+			where1 = PREFIX
+		elif front_restriction == 'noninternal':
+			where1 = FRONT_NOT_INTERNAL
+		else:
+			raise ValueError('Value {0} for front_restriction not allowed'.format(front_restriction))
+
+		if back_restriction is None:
+			where2 = BACK
+		elif back_restriction == 'anchored':
+			where2 = SUFFIX
+		elif back_restriction == 'noninternal':
+			where2 = BACK_NOT_INTERNAL
+		else:
+			raise ValueError(
+				'Value {0} for back_restriction not allowed'.format(back_restriction))
 		if require_both:
 			self._require_back_match = True
 			self._require_front_match = True
 		else:
-			self._require_front_match = front_anchored
-			self._require_back_match = back_anchored
+			self._require_front_match = front_restriction == 'anchored'
+			self._require_back_match = back_restriction == 'anchored'
 
 		# The following attributes are needed for the report
 		self.where = LINKED
 		self.name = _generate_adapter_name() if name is None else name
-		self.front_adapter = Adapter(front_sequence, where=where1, name=None, **kwargs)
-		self.back_adapter = Adapter(back_sequence, where=where2, name=None, **kwargs)
+		self.front_adapter = Adapter(front_sequence, where=where1, name=None, **front_parameters)
+		self.back_adapter = Adapter(back_sequence, where=where2, name=None, **back_parameters)
 
 	def enable_debug(self):
 		self.front_adapter.enable_debug()
