@@ -2,7 +2,6 @@ from __future__ import print_function, division, absolute_import
 
 import io
 import os
-import re
 import sys
 import copy
 import logging
@@ -18,7 +17,8 @@ from .modifiers import ZeroCapper
 from .report import Statistics
 from .filters import (Redirector, PairedRedirector, NoFilter, PairedNoFilter, InfoFileWriter,
 	RestFileWriter, WildcardFileWriter, TooShortReadFilter, TooLongReadFilter, NContentFilter,
-	DiscardTrimmedFilter, DiscardUntrimmedFilter, Demultiplexer, PairedEndDemultiplexer)
+	CasavaFilter, DiscardTrimmedFilter, DiscardUntrimmedFilter, Demultiplexer,
+	PairedEndDemultiplexer)
 from .seqio import read_chunks_from_file, read_paired_chunks
 
 logger = logging.getLogger()
@@ -33,7 +33,8 @@ class OutputFiles(object):
 	Files may also be None.
 	"""
 	# TODO interleaving for the other file pairs (too_short, too_long, untrimmed)?
-	def __init__(self,
+	def __init__(
+			self,
 			out=None,
 			out2=None,
 			untrimmed=None,
@@ -92,6 +93,14 @@ class Pipeline(object):
 		self._outfiles = None
 		self._demultiplexer = None
 
+		# Filter settings
+		self._minimum_length = None
+		self._maximum_length = None
+		self.max_n = None
+		self.discard_casava = False
+		self.discard_trimmed = False
+		self.discard_untrimmed = False
+
 	def set_input(self, file1, file2=None, qualfile=None, colorspace=False, fileformat=None,
 			interleaved=False):
 		self._reader = seqio.open(file1, file2, qualfile, colorspace, fileformat,
@@ -107,9 +116,7 @@ class Pipeline(object):
 		return seqio.open(file, file2, mode='w', qualities=self.uses_qualities,
 			colorspace=self._colorspace, **kwargs)
 
-	# TODO set max_n default to None
-	def set_output(self, outfiles, minimum_length=0, maximum_length=sys.maxsize,
-			max_n=-1, discard_trimmed=False, discard_untrimmed=False):
+	def set_output(self, outfiles):
 		self._filters = []
 		self._outfiles = outfiles
 		filter_wrapper = self._filter_wrapper()
@@ -121,24 +128,31 @@ class Pipeline(object):
 		if outfiles.wildcard:
 			self._filters.append(WildcardFileWriter(outfiles.wildcard))
 
-		too_short_writer = None
-		if minimum_length > 0:
-			if outfiles.too_short:
-				too_short_writer = self._open_writer(outfiles.too_short, outfiles.too_short2)
-			self._filters.append(
-				filter_wrapper(too_short_writer, TooShortReadFilter(minimum_length)))
+		# minimum length and maximum length
+		for lengths, file1, file2, filter_class in (
+				(self._minimum_length, outfiles.too_short, outfiles.too_short2, TooShortReadFilter),
+				(self._maximum_length, outfiles.too_long, outfiles.too_long2, TooLongReadFilter)
+		):
+			writer = None
+			if lengths is not None:
+				if file1:
+					writer = self._open_writer(file1, file2)
+				f1 = filter_class(lengths[0]) if lengths[0] is not None else None
+				if len(lengths) == 2 and lengths[1] is not None:
+					f2 = filter_class(lengths[1])
+				else:
+					f2 = None
+				self._filters.append(filter_wrapper(writer, filter=f1, filter2=f2))
 
-		too_long_writer = None
-		if maximum_length < sys.maxsize:
-			if outfiles.too_long:
-				too_long_writer = self._open_writer(outfiles.too_long, outfiles.too_long2)
-			self._filters.append(
-				filter_wrapper(too_long_writer, TooLongReadFilter(maximum_length)))
+		if self.max_n is not None:
+			f1 = f2 = NContentFilter(self.max_n)
+			self._filters.append(filter_wrapper(None, f1, f2))
 
-		if max_n != -1:
-			self._filters.append(filter_wrapper(None, NContentFilter(max_n)))
+		if self.discard_casava:
+			f1 = f2 = CasavaFilter()
+			self._filters.append(filter_wrapper(None, f1, f2))
 
-		if int(discard_trimmed) + int(discard_untrimmed) + int(outfiles.untrimmed is not None) > 1:
+		if int(self.discard_trimmed) + int(self.discard_untrimmed) + int(outfiles.untrimmed is not None) > 1:
 			raise ValueError('discard_trimmed, discard_untrimmed and outfiles.untrimmed must not '
 				'be set simultaneously')
 
@@ -149,13 +163,13 @@ class Pipeline(object):
 			# Set up the remaining filters to deal with --discard-trimmed,
 			# --discard-untrimmed and --untrimmed-output. These options
 			# are mutually exclusive in order to avoid brain damage.
-			if discard_trimmed:
-				self._filters.append(filter_wrapper(None, DiscardTrimmedFilter()))
-			elif discard_untrimmed:
-				self._filters.append(filter_wrapper(None, DiscardUntrimmedFilter()))
+			if self.discard_trimmed:
+				self._filters.append(filter_wrapper(None, DiscardTrimmedFilter(), DiscardTrimmedFilter()))
+			elif self.discard_untrimmed:
+				self._filters.append(filter_wrapper(None, DiscardUntrimmedFilter(), DiscardUntrimmedFilter()))
 			elif outfiles.untrimmed:
 				untrimmed_writer = self._open_writer(outfiles.untrimmed, outfiles.untrimmed2)
-				self._filters.append(filter_wrapper(untrimmed_writer, DiscardUntrimmedFilter()))
+				self._filters.append(filter_wrapper(untrimmed_writer, DiscardUntrimmedFilter(), DiscardUntrimmedFilter()))
 			self._filters.append(self._final_filter(outfiles))
 
 	def close(self):
@@ -216,10 +230,11 @@ class SingleEndPipeline(Pipeline):
 		for read in self._reader:
 			n += 1
 			total_bp += len(read.sequence)
+			matches = []
 			for modifier in self._modifiers:
-				read = modifier(read)
-			for filter in self._filters:
-				if filter(read):
+				read = modifier(read, matches)
+			for filter_ in self._filters:
+				if filter_(read, matches):
 					break
 		return (n, total_bp, None)
 
@@ -234,6 +249,24 @@ class SingleEndPipeline(Pipeline):
 		return Demultiplexer(outfiles.out, outfiles.untrimmed, qualities=self.uses_qualities,
 			colorspace=self._colorspace)
 
+	@property
+	def minimum_length(self):
+		return self._minimum_length
+
+	@minimum_length.setter
+	def minimum_length(self, value):
+		assert value is None or len(value) == 1
+		self._minimum_length = value
+
+	@property
+	def maximum_length(self):
+		return self._maximum_length
+
+	@maximum_length.setter
+	def maximum_length(self, value):
+		assert value is None or len(value) == 1
+		self._maximum_length = value
+
 
 class PairedEndPipeline(Pipeline):
 	"""
@@ -259,7 +292,7 @@ class PairedEndPipeline(Pipeline):
 	def add(self, modifier):
 		"""
 		Add a modifier for R1 and R2. If modify_first_read_only is True,
-		the modifier is *not* added for R2.
+		the modifier is not added for R2.
 		"""
 		self._modifiers.append(modifier)
 		if not self._modify_first_read_only:
@@ -285,13 +318,15 @@ class PairedEndPipeline(Pipeline):
 			n += 1
 			total1_bp += len(read1.sequence)
 			total2_bp += len(read2.sequence)
+			matches1 = []
+			matches2 = []
 			for modifier in self._modifiers:
-				read1 = modifier(read1)
+				read1 = modifier(read1, matches1)
 			for modifier in self._modifiers2:
-				read2 = modifier(read2)
+				read2 = modifier(read2, matches2)
 			for filter in self._filters:
 				# Stop writing as soon as one of the filters was successful.
-				if filter(read1, read2):
+				if filter(read1, read2, matches1, matches2):
 					break
 		return (n, total1_bp, total2_bp)
 
@@ -319,28 +354,23 @@ class PairedEndPipeline(Pipeline):
 			outfiles.untrimmed, outfiles.untrimmed2, qualities=self.uses_qualities,
 			colorspace=self._colorspace)
 
+	@property
+	def minimum_length(self):
+		return self._minimum_length
 
-def available_cpu_count():
-	"""
-	Return the number of available virtual or physical CPUs on this system.
-	The number of available CPUs can be smaller than the total number of CPUs
-	when the cpuset(7) mechanism is in use, as is the case on some cluster
-	systems.
+	@minimum_length.setter
+	def minimum_length(self, value):
+		assert value is None or len(value) == 2
+		self._minimum_length = value
 
-	Adapted from http://stackoverflow.com/a/1006301/715090
-	"""
-	try:
-		with open('/proc/self/status') as f:
-			status = f.read()
-		m = re.search(r'(?m)^Cpus_allowed:\s*(.*)$', status)
-		if m:
-			res = bin(int(m.group(1).replace(',', ''), 16)).count('1')
-			if res > 0:
-				return min(res, multiprocessing.cpu_count())
-	except IOError:
-		pass
+	@property
+	def maximum_length(self):
+		return self._maximum_length
 
-	return multiprocessing.cpu_count()
+	@maximum_length.setter
+	def maximum_length(self, value):
+		assert value is None or len(value) == 2
+		self._maximum_length = value
 
 
 def reader_process(file, file2, connections, queue, buffer_size, stdin_fd):
@@ -360,6 +390,7 @@ def reader_process(file, file2, connections, queue, buffer_size, stdin_fd):
 	and finally sends "poison pills" (the value -1) to all connections.
 	"""
 	if stdin_fd != -1:
+		sys.stdin.close()
 		sys.stdin = os.fdopen(stdin_fd)
 	try:
 		with xopen(file, 'rb') as f:
@@ -399,12 +430,11 @@ class WorkerProcess(Process):
 	To notify the reader process that it wants data, it puts its own identifier into the
 	need_work_queue before attempting to read data from the read_pipe.
 	"""
-	def __init__(self, id_, pipeline, filtering_options, input_path1, input_path2,
+	def __init__(self, id_, pipeline, input_path1, input_path2,
 			interleaved_input, orig_outfiles, read_pipe, write_pipe, need_work_queue):
 		super(WorkerProcess, self).__init__()
 		self._id = id_
 		self._pipeline = pipeline
-		self._filtering_options = filtering_options
 		self._input_path1 = input_path1
 		self._input_path2 = input_path2
 		self._interleaved_input = interleaved_input
@@ -452,7 +482,7 @@ class WorkerProcess(Process):
 
 				outfiles = OutputFiles(out=output, out2=output2, interleaved=self._orig_outfiles.interleaved)
 				self._pipeline.set_input(input, input2, interleaved=self._interleaved_input)
-				self._pipeline.set_output(outfiles, *self._filtering_options[0], **self._filtering_options[1])
+				self._pipeline.set_output(outfiles)
 				(n, bp1, bp2) = self._pipeline.process_reads()
 				cur_stats = Statistics()
 				cur_stats.collect(n, bp1, bp2, [], [], self._pipeline._filters)
@@ -507,7 +537,7 @@ class OrderedChunkWriter(object):
 
 class ParallelPipelineRunner(object):
 	"""
-	Wrap a SingleEndPipeline, running it in parallel
+	Run a Pipeline in parallel
 
 	- When set_input() is called, a reader process is spawned.
 	- When run() is called, as many worker processes as requested are spawned.
@@ -532,7 +562,6 @@ class ParallelPipelineRunner(object):
 		self._pipeline = pipeline
 		self._pipes = []  # the workers read from these
 		self._reader_process = None
-		self._filtering_options = None
 		self._outfiles = None
 		self._input_path1 = None
 		self._input_path2 = None
@@ -578,10 +607,9 @@ class ParallelPipelineRunner(object):
 			and not outfiles.demultiplex
 		)
 
-	def set_output(self, outfiles, *args, **kwargs):
+	def set_output(self, outfiles):
 		if not self.can_output_to(outfiles):
 			raise ValueError()
-		self._filtering_options = args, kwargs
 		self._outfiles = outfiles
 
 	def _start_workers(self):
@@ -590,8 +618,9 @@ class ParallelPipelineRunner(object):
 		for index in range(self._n_workers):
 			conn_r, conn_w = Pipe(duplex=False)
 			connections.append(conn_r)
-			worker = WorkerProcess(index, self._pipeline,
-				self._filtering_options, self._input_path1, self._input_path2,
+			worker = WorkerProcess(
+				index, self._pipeline,
+				self._input_path1, self._input_path2,
 				self._interleaved_input, self._outfiles,
 				self._pipes[index], conn_w, self._need_work_queue)
 			worker.daemon = True
