@@ -1,16 +1,18 @@
 # coding: utf-8
 from __future__ import print_function, division, absolute_import
-from nose.tools import raises, assert_raises
+from textwrap import dedent
+import pytest
 
 from cutadapt.seqio import Sequence
 from cutadapt.adapters import (Adapter, Match, ColorspaceAdapter, FRONT, BACK,
-	parse_braces, LinkedAdapter, AdapterStatistics)
+	parse_braces, LinkedAdapter, AdapterStatistics, AdapterParser, ANYWHERE)
 
 
 def test_issue_52():
 	adapter = Adapter(
 		sequence='GAACTCCAGTCACNNNNN',
 		where=BACK,
+		remove='suffix',
 		max_error_rate=0.12,
 		min_overlap=5,
 		read_wildcards=False,
@@ -47,6 +49,7 @@ def test_issue_80():
 	adapter = Adapter(
 		sequence="TCGTATGCCGTCTTC",
 		where=BACK,
+		remove='suffix',
 		max_error_rate=0.2,
 		min_overlap=3,
 		read_wildcards=False,
@@ -59,16 +62,16 @@ def test_issue_80():
 
 
 def test_str():
-	a = Adapter('ACGT', where=BACK, max_error_rate=0.1)
+	a = Adapter('ACGT', where=BACK, remove='suffix', max_error_rate=0.1)
 	str(a)
 	str(a.match_to(Sequence(name='seq', sequence='TTACGT')))
-	ca = ColorspaceAdapter('0123', where=BACK, max_error_rate=0.1)
+	ca = ColorspaceAdapter('0123', where=BACK, remove='suffix', max_error_rate=0.1)
 	str(ca)
 
 
-@raises(ValueError)
 def test_color():
-	ColorspaceAdapter('0123', where=FRONT, max_error_rate=0.1)
+	with pytest.raises(ValueError):
+		ColorspaceAdapter('0123', where=FRONT, max_error_rate=0.1)
 
 
 def test_parse_braces():
@@ -87,13 +90,15 @@ def test_parse_braces():
 def test_parse_braces_fail():
 	for expression in ['{', '}', '{}', '{5', '{1}', 'A{-7}', 'A{', 'A{1', 'N{7', 'AN{7', 'A{4{}',
 			'A{4}{3}', 'A{b}', 'A{6X}', 'A{X6}']:
-		assert_raises(ValueError, lambda: parse_braces(expression))
+		with pytest.raises(ValueError):
+			parse_braces(expression)
 
 
 def test_linked_adapter():
-	linked_adapter = LinkedAdapter('AAAA', 'TTTT', min_overlap=4)
+	linked_adapter = LinkedAdapter('AAAA', 'TTTT',
+		front_parameters={'min_overlap': 4}, back_parameters={'min_overlap': 3})
 	assert linked_adapter.front_adapter.min_overlap == 4
-	assert linked_adapter.back_adapter.min_overlap == 4
+	assert linked_adapter.back_adapter.min_overlap == 3
 
 	sequence = Sequence(name='seq', sequence='AAAACCCCCTTTT')
 	trimmed = linked_adapter.match_to(sequence).trimmed()
@@ -187,5 +192,92 @@ def test_add_adapter_statistics():
 def test_issue_265():
 	"""Crash when accessing the matches property of non-anchored linked adapters"""
 	s = Sequence('name', 'AAAATTTT')
-	la = LinkedAdapter('GGG', 'TTT', front_anchored=False, back_anchored=False)
+	la = LinkedAdapter('GGG', 'TTT', front_restriction=None, back_restriction=None)
 	assert la.match_to(s).matches == 3
+
+
+def test_parse_file_notation(tmpdir):
+	tmp_path = str(tmpdir.join('adapters.fasta'))
+	with open(tmp_path, 'w') as f:
+		f.write(dedent(""">first_name
+			ADAPTER1
+			>second_name
+			ADAPTER2
+			"""))
+	parser = AdapterParser(
+		colorspace=False, max_error_rate=0.2, min_overlap=4, read_wildcards=False,
+		adapter_wildcards=False, indels=False)
+
+	adapters = list(parser.parse('file:' + tmp_path, cmdline_type='back'))
+	assert len(adapters) == 2
+	assert adapters[0].name == 'first_name'
+	assert adapters[0].sequence == 'ADAPTER1'
+	assert adapters[1].name == 'second_name'
+	assert adapters[1].sequence == 'ADAPTER2'
+	for a in adapters:
+		assert a.max_error_rate == 0.2
+		assert a.min_overlap == 4
+		assert not a.read_wildcards
+		assert not a.adapter_wildcards
+		assert not a.indels
+
+
+def test_parse_not_linked():
+	p = AdapterParser._parse_not_linked
+	assert p('A', 'front') == (None, None, 'A', None, {})
+	assert p('A', 'back') == (None, None, 'A', None, {})
+	assert p('A', 'anywhere') == (None, None, 'A', None, {})
+	assert p('^A', 'front') == (None, 'anchored', 'A', None, {})
+	assert p('XXXA', 'front') == (None, 'noninternal', 'A', None, {})
+	assert p('A$', 'back') == (None, None, 'A', 'anchored', {})
+	assert p('AXXXX', 'back') == (None, None, 'A', 'noninternal', {})
+	assert p('a_name=ADAPT', 'front') == ('a_name', None, 'ADAPT', None, {})
+
+
+def test_parse_parameters():
+	p = AdapterParser._parse_parameters
+	assert p('e=0.1') == {'max_error_rate': 0.1}
+	assert p('error_rate=0.1') == {'max_error_rate': 0.1}
+	assert p('o=5') == {'min_overlap': 5}
+	assert p('min_overlap=5') == {'min_overlap': 5}
+	assert p('o=7; e=0.4') == {'min_overlap': 7, 'max_error_rate': 0.4}
+	assert p('anywhere') == {'anywhere': True}
+
+	with pytest.raises(ValueError):
+		p('e=hallo')
+	with pytest.raises(KeyError):
+		p('bla=0.1')
+	with pytest.raises(ValueError):
+		p('e=')
+
+
+def test_parse_with_parameters():
+	parser = AdapterParser(
+		colorspace=False, max_error_rate=0.2, min_overlap=4, read_wildcards=False,
+		adapter_wildcards=False, indels=False)
+	a = parser._parse('ACGTACGT; e=0.15', 'front')
+	assert a.max_error_rate == 0.15
+	assert a.min_overlap == 4
+
+	a = parser._parse('ACGTAAAA; o=5; e=0.11', 'back')
+	assert a.max_error_rate == 0.11
+	assert a.min_overlap == 5
+
+	for spec in ('thename=ACG;e=0.15 ... TGT;e=0.17', 'thename=ACG;e=0.15...TGT;e=0.17'):
+		a = parser._parse(spec, 'back')
+		assert isinstance(a, LinkedAdapter)
+		assert a.front_adapter.max_error_rate == 0.15
+		assert a.back_adapter.max_error_rate == 0.17
+
+
+def test_anywhere_parameter():
+	parser = AdapterParser(colorspace=False, max_error_rate=0.2, min_overlap=4, read_wildcards=False,
+		adapter_wildcards=False, indels=True)
+	adapter = list(parser.parse('CTGAAGTGAAGTACACGGTT;anywhere', 'back'))[0]
+	assert adapter.remove == 'suffix'
+	assert adapter.where == ANYWHERE
+	read = Sequence('foo1', 'TGAAGTACACGGTTAAAAAAAAAA')
+	from cutadapt.modifiers import AdapterCutter
+	cutter = AdapterCutter([adapter])
+	trimmed_read = cutter(read, [])
+	assert trimmed_read.sequence == ''
