@@ -10,12 +10,15 @@ import subprocess
 import sys
 import tempfile
 
-from nose.tools import raises
+import pytest
 
 from cutadapt.__main__ import main
-from cutadapt.compat import StringIO
+from cutadapt.compat import StringIO, PY3
 from utils import run, assert_files_equal, datapath, cutpath, redirect_stderr, temporary_path
 
+import pytest_timeout as _unused
+del _unused
+
 
 def test_example():
 	run('-N -b ADAPTER', 'example.fa', 'example.fa')
@@ -156,11 +159,26 @@ def test_polya_brace_notation():
 	run("-m 24 -O 10 -a A{35}", "polya.fasta", "polya.fasta")
 
 
+# the same as --action=none
+def test_no_trim():
+	run("--no-trim --discard-untrimmed -a CCCTAGTTAAAC", 'no-trim.fastq', 'small.fastq')
+
+
+def test_action_none():
+	run("--action=none --discard-untrimmed -a CCCTAGTTAAAC", 'no-trim.fastq', 'small.fastq')
+
+
+# the same as --action=mask
 def test_mask_adapter():
 	"""mask adapter with N (reads maintain the same length)"""
 	run("-b CAAG -n 3 --mask-adapter", "anywhere_repeat.fastq", "anywhere_repeat.fastq")
 
 
+def test_action_mask():
+	"""mask adapter with N (reads maintain the same length)"""
+	run("-b CAAG -n 3 --action=mask", "anywhere_repeat.fastq", "anywhere_repeat.fastq")
+
+
 def test_gz_multiblock():
 	"""compressed gz file with multiple blocks (created by concatenating two .gz files)"""
 	run("-b TTAGACATATCTCCGTCG", "small.fastq", "multiblock.fastq.gz")
@@ -291,10 +309,6 @@ def test_adapter_with_u():
 	run("-a GCCGAACUUCUUAGACUGCCUUAAGGACGU", "illumina.fastq", "illumina.fastq.gz")
 
 
-def test_no_trim():
-	run("--no-trim --discard-untrimmed -a CCCTAGTTAAAC", 'no-trim.fastq', 'small.fastq')
-
-
 def test_bzip2():
 	run('-b TTAGACATATCTCCGTCG', 'small.fastq', 'small.fastq.bz2')
 
@@ -313,22 +327,22 @@ except ImportError:
 	pass
 
 
-@raises(SystemExit)
 def test_qualfile_only():
-	with redirect_stderr():
-		main(['file.qual'])
+	with pytest.raises(SystemExit):
+		with redirect_stderr():
+			main(['file.qual'])
 
 
-@raises(SystemExit)
 def test_no_args():
-	with redirect_stderr():
-		main([])
+	with pytest.raises(SystemExit):
+		with redirect_stderr():
+			main([])
 
 
-@raises(SystemExit)
 def test_two_fastqs():
-	with redirect_stderr():
-		main([datapath('paired.1.fastq'), datapath('paired.2.fastq')])
+	with pytest.raises(SystemExit):
+		with redirect_stderr():
+			main([datapath('paired.1.fastq'), datapath('paired.2.fastq')])
 
 
 def test_anchored_no_indels():
@@ -346,10 +360,10 @@ def test_anchored_no_indels_wildcard_adapt():
 	run('-g ^TTAGACANAT --no-indels -e 0.1', 'anchored_no_indels.fasta', 'anchored_no_indels.fasta')
 
 
-@raises(SystemExit)
 def test_non_iupac_characters():
-	with redirect_stderr():
-		main(['-a', 'ZACGT', datapath('small.fastq')])
+	with pytest.raises(SystemExit):
+		with redirect_stderr():
+			main(['-a', 'ZACGT', datapath('small.fastq')])
 
 
 def test_unconditional_cut_front():
@@ -457,22 +471,22 @@ def test_linked_discard_untrimmed_g():
 	run('-g AAAAAAAAAA...TTTTTTTTTT --discard-untrimmed', 'linked-discard-g.fasta', 'linked.fasta')
 
 
-@raises(SystemExit)
 def test_linked_anywhere():
-	with redirect_stderr():
-		main(['-b', 'AAA...TTT', datapath('linked.fasta')])
+	with pytest.raises(SystemExit):
+		with redirect_stderr():
+			main(['-b', 'AAA...TTT', datapath('linked.fasta')])
 
 
-@raises(SystemExit)
 def test_anywhere_anchored_5p():
-	with redirect_stderr():
-		main(['-b', '^AAA', datapath('small.fastq')])
+	with pytest.raises(SystemExit):
+		with redirect_stderr():
+			main(['-b', '^AAA', datapath('small.fastq')])
 
 
-@raises(SystemExit)
 def test_anywhere_anchored_3p():
-	with redirect_stderr():
-		main(['-b', 'TTT$', datapath('small.fastq')])
+	with pytest.raises(SystemExit):
+		with redirect_stderr():
+			main(['-b', 'TTT$', datapath('small.fastq')])
 
 
 def test_fasta():
@@ -492,5 +506,46 @@ def test_length():
 	run('--length 5', 'shortened.fastq', 'small.fastq')
 
 
+def test_negative_length():
+	run('--length -5', 'shortened-negative.fastq', 'small.fastq')
+
+
 def test_run_cutadapt_process():
 	subprocess.check_call(['cutadapt', '--version'])
+
+
+@pytest.mark.timeout(0.5)
+def test_issue_296(tmpdir):
+	# Hang when using both --no-trim and --info-file together
+	info_path = str(tmpdir.join('info.txt'))
+	reads_path = str(tmpdir.join('reads.fasta'))
+	out_path = str(tmpdir.join('out.fasta'))
+	with open(reads_path, 'w') as f:
+		f.write('>read\nCACAAA\n')
+	main(['--info-file', info_path, '--no-trim', '-g', 'TTTCAC', '-o', out_path, reads_path])
+	# Output should be unchanged because of --no-trim
+	assert_files_equal(reads_path, out_path)
+
+
+def test_xadapter():
+	run('-g XTCCGAATAGA', 'xadapter.fasta', 'xadapterx.fasta')
+
+
+def test_adapterx():
+	run('-a TCCGAATAGAX', 'adapterx.fasta', 'xadapterx.fasta')
+
+
+def test_discard_casava():
+	run('--discard-casava', 'casava.fastq', 'casava.fastq')
+
+
+@pytest.mark.xfail  # FIXME
+def test_underscore():
+	"""File name ending in _fastq.gz (issue #275)"""
+	run('-b TTAGACATATCTCCGTCG', 'small.fastq', 'underscore_fastq.gz')
+
+
+if PY3:
+	def test_cores_autodetect():
+		# Just make sure that it runs; functionality is not tested
+		run('--cores 0 -b TTAGACATATCTCCGTCG', 'small.fastq', 'underscore_fastq.gz')
